@@ -100,7 +100,6 @@ typedef struct {
 } IcsTextMenuEntry;
 
 IcsTextMenuEntry icsTextMenuEntry[ICS_TEXT_MENU_SIZE];
-char dataDir[MSG_SIZ] = DATADIR;
 
 int junk;
 int saveDate;
@@ -223,8 +222,7 @@ ArgDescriptor argDescriptors[] = {
   { "flashRate", ArgInt, (void *) &appData.flashRate, XBOARD, (ArgIniType) FLASH_RATE },
   { "pieceImageDirectory", ArgFilename, (void *) &appData.pieceDirectory, TRUE, (ArgIniType) "" },
   { "pid", ArgFilename, (void *) &appData.pieceDirectory, FALSE, INVALID },
-  { "trueColors", ArgBoolean, (void *) &appData.trueColors, XBOARD, (ArgIniType) (2*!XBOARD) },
-  { "jewelled", ArgInt, (void *) &appData.jewelled, FALSE, (ArgIniType) -1 },
+  { "trueColors", ArgBoolean, (void *) &appData.trueColors, TRUE, (ArgIniType) FALSE },
   { "soundDirectory", ArgFilename, (void *) &appData.soundDirectory, XBOARD, (ArgIniType) "" },
   { "msLoginDelay", ArgInt, (void *) &appData.msLoginDelay, XBOARD, (ArgIniType) MS_LOGIN_DELAY },
   { "pasteSelection", ArgBoolean, (void *) &appData.pasteSelection, XBOARD, (ArgIniType) FALSE },
@@ -535,7 +533,6 @@ ArgDescriptor argDescriptors[] = {
   { "secondScoreAbs", ArgBoolean, (void *) &appData.secondScoreIsAbsolute, FALSE, (ArgIniType) FALSE },
   { "pgnExtendedInfo", ArgBoolean, (void *) &appData.saveExtendedInfoInPGN, TRUE, (ArgIniType) FALSE },
   { "hideThinkingFromHuman", ArgBoolean, (void *) &appData.hideThinkingFromHuman, TRUE, (ArgIniType) FALSE },
-  { "pgnTimeLeft", ArgBoolean, (void *) &appData.cumulativeTimePGN, TRUE, (ArgIniType) FALSE },
   { "liteBackTextureFile", ArgFilename, (void *) &appData.liteBackTextureFile, TRUE, (ArgIniType) "" },
   { "lbtf", ArgFilename, (void *) &appData.liteBackTextureFile, FALSE, INVALID },
   { "darkBackTextureFile", ArgFilename, (void *) &appData.darkBackTextureFile, TRUE, (ArgIniType) "" },
@@ -618,7 +615,6 @@ ArgDescriptor argDescriptors[] = {
   { "inscriptions", ArgString, (void *) &appData.inscriptions, FALSE, (ArgIniType) "" },
   { "autoInstall", ArgString, (void *) &appData.autoInstall, XBOARD, (ArgIniType) "" },
   { "fixedSize", ArgBoolean, (void *) &appData.fixedSize, TRUE, (ArgIniType) FALSE },
-  { "showMoveTime", ArgBoolean, (void *) &appData.moveTime, TRUE, (ArgIniType) FALSE },
 
   // [HGM] tournament options
   { "tourneyFile", ArgFilename, (void *) &appData.tourneyFile, FALSE, (ArgIniType) "" },
@@ -715,11 +711,6 @@ ArgDescriptor argDescriptors[] = {
   { "firstDrawDepth", ArgInt, (void *) &appData.drawDepth[0], FALSE, (ArgIniType) 0 },
   { "secondDrawDepth", ArgInt, (void *) &appData.drawDepth[1], FALSE, (ArgIniType) 0 },
   { "memoHeaders", ArgBoolean, (void *) &appData.headers, TRUE, (ArgIniType) FALSE },
-  { "startupMessage", ArgString, (void *) &appData.message, FALSE, (ArgIniType) "" },
-  { "messageSuppress", ArgString, (void *) &appData.suppress, XBOARD, (ArgIniType) "" },
-  { "fen", ArgString, (void *) &appData.fen, FALSE, (ArgIniType) "" },
-  { "men", ArgString, (void *) &appData.men, FALSE, (ArgIniType) "" },
-  { "analysisBell", ArgInt, (void *) &appData.analysisBell, TRUE, (ArgIniType) 0 },
 
 #if ZIPPY
   { "zippyTalk", ArgBoolean, (void *) &appData.zippyTalk, FALSE, (ArgIniType) ZIPPY_TALK },
@@ -938,7 +929,7 @@ ParseSettingsFile(char *name, char **addr)
 	MySearchPath(installDir, buf, fullname); // first look in user's own files
 	f = fopen(fullname, "r");
 	if(f == NULL) {
-	    snprintf(buf, MSG_SIZ, "%s/themes/conf", dataDir);
+	    snprintf(buf, MSG_SIZ, "%s/themes/conf", DATADIR);
 	    MySearchPath(buf, name, fullname); // also look in standard place
 	    f = fopen(fullname, "r");
 	}
@@ -963,7 +954,7 @@ ParseArgs(GetFunc get, void *cl)
   char argValue[MAX_ARG_LEN];
   ArgDescriptor *ad;
   char start;
-  char *q, *r, *s;
+  char *q;
   int i, octval;
   char ch;
   int posarg = 4; // default is game file
@@ -1178,7 +1169,7 @@ ParseArgs(GetFunc get, void *cl)
     case ArgFilename:
       if(argValue[0] == '~' && argValue[1] == '~') {
         char buf[4*MSG_SIZ]; // expand ~~
-        snprintf(buf, 4*MSG_SIZ, "%s%s", dataDir, argValue+2);
+        snprintf(buf, 4*MSG_SIZ, "%s%s", DATADIR, argValue+2);
         ASSIGN(*(char **) ad->argLoc, buf);
         break;
       }
@@ -1243,29 +1234,10 @@ ParseArgs(GetFunc get, void *cl)
 
     case ArgInstall:
       q = *(char **) ad->argLoc;
-      r = NULL; s = argValue;
-      if(argValue[0] == '#') { // group specification
-        r = strstr(argValue, "\\n");
-        if(r) *r++ = '\n', *r++ = NULLCHAR, s = r, r = argValue; // split s into line-to-add (s) and group (r)
-      }
-      if((saveDate == 0 || saveDate - dateStamp < 0) && !strstr(q, s) ) { // not seen before, and line does not occur yet
-        int l = strlen(q) + strlen(s);
-        if(r) { // must be put in group r
-          char *p = strstr(q, r);
-          if(p) { // group already exists
-            p += strlen(r) - 1; // determine insertion point (immediately after group header line)
-            *(char **) ad->argLoc = malloc(l+2);
-            *p++ = NULLCHAR; // spit old value (q) at insertion point into q and p
-            snprintf(*(char **) ad->argLoc, l+2, "%s\n%s\n%s", q, s, p); // insert (with newline)
-          } else { // group did not exist, create at end
-            l += strlen(r) + 8;
-            *(char **) ad->argLoc = malloc(l);
-            snprintf(*(char **) ad->argLoc, l, "%s%s%s\n# end\n", q, r, s);
-          }
-        } else { // no group, just add line at end
-          *(char **) ad->argLoc = malloc(l+2);
-          snprintf(*(char **) ad->argLoc, l+2, "%s%s\n", q, s);
-        }
+      if((saveDate == 0 || saveDate - dateStamp < 0) && !strstr(q, argValue) ) {
+        int l = strlen(q) + strlen(argValue);
+        *(char **) ad->argLoc = malloc(l+2);
+        snprintf(*(char **) ad->argLoc, l+2, "%s%s\n", q, argValue);
         free(q);
       }
       break;
@@ -1438,11 +1410,7 @@ InitAppData(char *lpCmdLine)
      appData.NrRanks > BOARD_RANKS   )
       DisplayFatalError("Recompile with BOARD_RANKS or BOARD_FILES, to support this size", 0, 2);
 
-  if(!*appData.secondChessProgram) { // [HGM] scp defaults to fcp
-    ASSIGN(appData.secondChessProgram, appData.firstChessProgram);
-    ASSIGN(appData.secondDirectory, appData.firstDirectory);
-    appData.secondIsUCI = appData.firstIsUCI; // copy type too!
-  }
+  if(!*appData.secondChessProgram) { ASSIGN(appData.secondChessProgram, appData.firstChessProgram); } // [HGM] scp defaults to fcp
 
   /* [HGM] After parsing the options from the .ini file, and overruling them
    * with options from the command line, we now make an even higher priority
@@ -1551,10 +1519,11 @@ SaveSettings(char* name)
   ArgDescriptor *ad;
   char dir[MSG_SIZ], buf[MSG_SIZ];
   int mps = appData.movesPerSession;
+  TimeMark now;
 
   if (!MainWindowUp() && !autoClose) return;
 
-  saveDate = programStartTime.sec;
+  GetTimeMark(&now); saveDate = now.sec;
 
   GetCurrentDirectory(MSG_SIZ, dir);
   if(MySearchPath(installDir, name, buf)) {
@@ -1657,8 +1626,8 @@ SaveSettings(char* name)
       { char buf[MSG_SIZ];
         snprintf(buf, MSG_SIZ, "%s", *(char**)ad->argLoc);
 #ifdef OSXAPP
-        if(strstr(buf, dataDir) == buf)
-          snprintf(buf, MSG_SIZ, "~~%s", *(char**)ad->argLoc + strlen(dataDir));
+        if(strstr(buf, DATADIR) == buf)
+          snprintf(buf, MSG_SIZ, "~~%s", *(char**)ad->argLoc + strlen(DATADIR));
 #endif
         if (strchr(buf, '\"')) {
           fprintf(f, OPTCHAR "%s" SEPCHAR "'%s'\n", ad->argName, buf);
