@@ -80,7 +80,6 @@ extern Boolean mcMode;
 extern int dragging;
 extern char variantError[];
 extern char lastTalker[];
-extern int transparency[];
 
 void MarkMenuItem P((char *menuRef, int state));
 char *CmailMsg P((void));
@@ -429,7 +428,6 @@ char *SupportedVariant P((char *list, VariantClass v, int w, int h, int s, int p
 char *CollectPieceDescriptors P((void));
 void RefreshSettingsDialog P((ChessProgramState *cps, int val));
 void StartChessProgram P((ChessProgramState *cps));
-void SendToICS P((char *s));
 int PosFlags P((int n));
 
 
