@@ -5,7 +5,8 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016 Free
+ * Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -186,7 +187,7 @@ typedef char *String;
 #define EP_FILE      CASTLING][(BOARD_FILES-3) /* [HGM] in upper rank        */
 #define EP_STATUS    CASTLING][(BOARD_FILES-2) /* [HGM] in upper rank        */
 #define HOLDINGS_SET CASTLING][(BOARD_FILES-1) /* [HGM] in upper-right corner*/
-#define ONE          ('1'-(BOARD_HEIGHT==10))  /* [HGM] foremost board rank  */
+#define ONE          ('1'-(BOARD_HEIGHT==10)-appData.rankOffset)  /* [HGM] foremost board rank  */
 #define AAA          ('a'-BOARD_LEFT)          /* [HGM] leftmost board file  */
 #define VIRGIN_W                 1             /* [HGM] flags in Board[VIRGIN][X] */
 #define VIRGIN_B                 2
@@ -226,6 +227,7 @@ typedef char *String;
 #define BELLCHAR                '\007'
 #define NULLCHAR                '\000'
 #define FEATURE_TIMEOUT         10000 /*ms*/
+#define MATE_SCORE              100000
 
 #define CLOCK_FONT 0
 #define MESSAGE_FONT 1
@@ -293,18 +295,26 @@ typedef enum {
     WhiteFerz, WhiteAlfil, WhiteAngel, WhiteMarshall, WhiteWazir, WhiteMan,
     WhiteCannon, WhiteNightrider, WhiteCardinal, WhiteDragon, WhiteGrasshopper,
     WhiteSilver, WhiteFalcon, WhiteLance, WhiteCobra, WhiteUnicorn, WhiteLion,
-    WhiteTokin, WhiteDagger, WhitePCardinal, WhitePDragon, WhiteCat,
-    WhitePSword, WhiteMonarch, WhiteMother, WhiteNothing, WhitePRook, WhitePDagger,
-    WhiteDolphin, WhiteStag, WhiteHorned, WhiteEagle, WhiteSword,
-    WhiteCrown, WhiteHCrown, WhiteHorse, WhiteDrunk, WhitePBishop, WhiteKing,
+    WhiteSword, WhiteZebra, WhiteCamel, WhiteTower, WhiteWolf,
+    WhiteHat, WhiteDuck, WhiteAmazon, WhiteFlying, WhiteGnu, WhiteCub,
+    WhiteShield, WhiteHorse, WhiteWizard, WhiteCopper, WhiteIron,
+    WhiteViking, WhiteFlag, WhiteAxe, WhiteDolphin, WhiteCat, WhiteClaw,
+    WhiteWheel, WhiteButterfly, WhitePBishop, WhitePRook, WhiteHCrown,
+    WhiteShierd, WhiteMonarch, WhiteMother, WhiteNothing, WhiteDrunk, WhiteWheer,
+    WhiteTokin, WhitePKnight, WhitePCardinal, WhitePDragon, WhitePLance,
+    WhitePSilver, WhiteDagger, WhitePSword, WhitePDagger, WhiteCrown, WhiteKing,
     BlackPawn, BlackKnight, BlackBishop, BlackRook, BlackQueen,
     BlackFerz, BlackAlfil, BlackAngel, BlackMarshall, BlackWazir, BlackMan,
     BlackCannon, BlackNightrider, BlackCardinal, BlackDragon, BlackGrasshopper,
     BlackSilver, BlackFalcon, BlackLance, BlackCobra, BlackUnicorn, BlackLion,
-    BlackTokin, BlackDagger, BlackPCardinal, BlackPDragon, BlackCat,
-    BlackPSword, BlackMonarch, BlackMother, BlackNothing, BlackPRook, BlackPDagger,
-    BlackDolphin, BlackStag, BlackHorned, BlackEagle, BlackSword,
-    BlackCrown, BlackHCrown, BlackHorse, BlackDrunk, BlackPBishop, BlackKing,
+    BlackSword, BlackZebra, BlackCamel, BlackTower, BlackWolf,
+    BlackHat, BlackDuck, BlackAmazon, BlackFlying, BlackGnu, BlackCub,
+    BlackShield, BlackHorse, BlackWizard, BlackCopper, BlackIron,
+    BlackViking, BlackFlag, BlackAxe, BlackDolphin, BlackCat, BlackClaw,
+    BlackWheel, BlackButterfly, BlackPBishop, BlackPRook, BlackHCrown,
+    BlackShierd, BlackMonarch, BlackMother, BlackNothing, BlackDrunk, BlackWheer,
+    BlackTokin, BlackPKnight, BlackPCardinal, BlackPDragon, BlackPLance,
+    BlackPSilver, BlackDagger, BlackPSword, BlackPDagger, BlackCrown, BlackKing,
     EmptySquare, DarkSquare,
     NoRights, // [HGM] gamestate: for castling rights hidden in board[CASTLING]
     ClearBoard, WhitePlay, BlackPlay, PromotePiece, DemotePiece /*for use on EditPosition menus*/
@@ -313,11 +323,12 @@ typedef enum {
 /* [HGM] some macros that can be used as prefixes to convert piece types */
 #define WHITE_TO_BLACK (int)BlackPawn - (int)WhitePawn + (int)
 #define BLACK_TO_WHITE (int)WhitePawn - (int)BlackPawn + (int)
-#define PROMOTED       (int)WhiteDragon - (int)WhiteRook + (int)
-#define DEMOTED        (int)WhiteRook - (int)WhiteDragon + (int)
+#define PROMO          (int)WhiteDragon - (int)WhiteRook + (int)
+#define PROMOTED(X)    (promoPartner[X])
+#define DEMOTED(X)     (promoPartner[X])
 #define SHOGI          (int)EmptySquare + (int)
-#define CHUPROMOTED    ((int)WhitePDragon - (int)WhiteDragon)*(gameInfo.variant == VariantChu) + PROMOTED
-#define CHUDEMOTED     ((int)WhiteDragon - (int)WhitePDragon)*(gameInfo.variant == VariantChu) + DEMOTED
+#define CHUPROMOTED(X) (promoPartner[X])
+#define CHUDEMOTED(X)  (promoPartner[X])
 #define IS_SHOGI(V)    ((V) == VariantShogi || (V) == VariantChu)
 #define IS_LION(V)     ((V) == WhiteLion || (V) == BlackLion)
 
@@ -486,6 +497,7 @@ typedef struct {
     char *pgnName[ENGINES];
     Boolean firstPlaysBlack;
     Boolean noChessProgram;
+    char *positionDir;
     char *host[ENGINES];
     char *themeNames;
     char *pieceDirectory;
@@ -521,6 +533,7 @@ typedef struct {
     Boolean fischerCastling;/* [HGM] fischer: allow Fischr castling in any variant */
     Boolean matchMode;
     int matchGames;
+    Boolean epd;
     Boolean monoMode;
     Boolean debugMode;
     Boolean clockMode;
@@ -688,6 +701,7 @@ typedef struct {
     /* [HGM] Board size */
     int NrFiles;
     int NrRanks;
+    int rankOffset;
     int holdingsSize;
     int matchPause;
     char * pieceToCharTable;
@@ -777,6 +791,7 @@ typedef struct {
     Boolean autoCopyPV;
     Boolean pieceMenu;
     Boolean sweepSelect;
+    Boolean monoMouse;
     Boolean whitePOV;
     Boolean scoreWhite;
     Boolean pvSAN[ENGINES];
