@@ -131,34 +131,9 @@ void
 SwitchWindow (int main)
 {
     currBoard = (main ? &mainOptions[W_BOARD] : &dualOptions[3]);
-//    CsBoardWindow = DRAWABLE(currBoard);
+    csBoardWindow = DRAWABLE(currBoard);
 }
 
-
-static void
-NewCanvas (Option *graph)
-{
-	cairo_t *cr;
-	int w = graph->max, h = graph->value;
-	if(graph->choice) cairo_surface_destroy((cairo_surface_t *) graph->choice);
-	graph->choice = (char**) cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
-	// paint white, to prevent weirdness when people maximize window and drag pieces over space next to board
-	cr = cairo_create ((cairo_surface_t *) graph->choice);
-	cairo_rectangle (cr, 0, 0, w, h);
-	cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0);
-	cairo_fill(cr);
-	cairo_destroy (cr);
-	graph->min &= ~REPLACE;
-}
-
-static cairo_surface_t *
-CsBoardWindow (Option *opt)
-{   // test before every draw event if we need to resize the canvas
-    if(opt->min & REPLACE) NewCanvas(opt);
-    return DRAWABLE(opt);
-}
-
-
 void
 SelectPieces(VariantClass v)
 {
@@ -187,6 +162,20 @@ SelectPieces(VariantClass v)
 	if(v == VariantChuChess) {
 	   pngPieceBitmaps[i][(int)WhiteNightrider] = pngPieceBitmaps2[i][(int)WhiteLion];
 	}
+	if(v == VariantChu) {
+	   pngPieceBitmaps[i][(int)WhiteNightrider] = pngPieceBitmaps2[i][(int)WhiteClaw];
+	   pngPieceBitmaps[i][(int)WhiteClaw]    = pngPieceBitmaps2[i][(int)WhiteNightrider];
+	   pngPieceBitmaps[i][(int)WhiteUnicorn] = pngPieceBitmaps2[i][(int)WhiteCat];
+	   pngPieceBitmaps[i][(int)WhiteSilver]  = pngPieceBitmaps2[i][(int)WhiteSword];
+	   pngPieceBitmaps[i][(int)WhiteFalcon]  = pngPieceBitmaps2[i][(int)WhiteDagger];
+	   pngPieceBitmaps[i][(int)WhiteCat]     = pngPieceBitmaps2[i][(int)WhiteUnicorn];
+	   pngPieceBitmaps[i][(int)WhiteSword]   = pngPieceBitmaps2[i][(int)WhiteSilver];
+	   pngPieceBitmaps[i][(int)WhiteDagger]  = pngPieceBitmaps2[i][(int)WhiteFalcon];
+	   pngPieceBitmaps[i][(int)WhiteMan]     = pngPieceBitmaps2[i][(int)WhiteCopper];
+	   pngPieceBitmaps[i][(int)WhiteCopper]  = pngPieceBitmaps2[i][(int)WhiteMan];
+	   pngPieceBitmaps[i][(int)WhiteAxe]     = pngPieceBitmaps2[i][(int)WhiteCannon];
+	   pngPieceBitmaps[i][(int)WhiteCannon]  = pngPieceBitmaps2[i][(int)WhiteAxe];
+	}
     }
 }
 
@@ -206,7 +195,7 @@ InitDrawingSizes (BoardSize boardSize, int flags)
 	squareSize = ((squareSize + lineGap) * oldNrOfFiles + 0.5*BOARD_WIDTH) / BOARD_WIDTH; // keep total width fixed
 	if(appData.overrideLineGap < 0) lineGap = squareSize < 37 ? 1 : squareSize < 59 ? 2 : squareSize < 116 ? 3 : 4;
         squareSize -= lineGap;
-	CreatePNGPieces(appData.pieceDirectory);
+	CreatePNGPieces();
         CreateGrid();
     }
     oldNrOfFiles = BOARD_WIDTH;
@@ -255,8 +244,6 @@ ExposeRedraw (Option *graph, int x, int y, int w, int h)
     cairo_destroy(cr);
 }
 
-static int modV[2], modH[2];
-
 static void
 CreatePNGBoard (char *s, int kind)
 {
@@ -274,15 +261,13 @@ CreatePNGBoard (char *s, int kind)
 	    useTexture |= kind + 1; pngOriginalBoardBitmap[kind] = img;
 	    w = textureW[kind] = cairo_image_surface_get_width (img);
 	    h = textureH[kind] = cairo_image_surface_get_height (img);
-	    transparency[kind] = cairo_image_surface_get_format (img) == CAIRO_FORMAT_ARGB32;
-	    n[kind] = 1.; modV[kind] = modH[kind] = -1;
+	    n[kind] = 1.;
 	    while((q = strchr(p+1, '-'))) p = q; // find last '-'
 	    if(strlen(p) < 11 && sscanf(p, "-%dx%d.pn%c", &f, &r, &c) == 3 && c == 'g') {
 		if(f == 0 || r == 0) f = BOARD_WIDTH, r = BOARD_HEIGHT; // 0x0 means 'fits any', so make it fit
 		textureW[kind] = (w*BOARD_WIDTH)/f; // sync cutting locations with square pattern
 		textureH[kind] = (h*BOARD_HEIGHT)/r;
-		n[kind] = (r*squareSize + 0.99)/h;  // scale to make it fit exactly vertically
-		modV[kind] = r; modH[kind] = f;
+		n[kind] = r*squareSize/h; // scale to make it fit exactly vertically
 	    } else
 	    if((p = strstr(s, "xq")) && (p == s || p[-1] == '/')) { // assume full-board image for Xiangqi
 		while(0.8*squareSize*BOARD_WIDTH > n[kind]*w || 0.8*squareSize*BOARD_HEIGHT > n[kind]*h) n[kind]++;
@@ -338,15 +323,6 @@ LoadSVG (char *dir, int color, int piece, int retry)
 
     if(!svg && *dir) {
       svg = rsvg_handle_new_from_file(buf, &svgerror);
-      if(!svg) { // failed! If -pid name starts with "sub_" we try to load the piece from the parent directory
-	char *p = buf, *q;
-	safeStrCpy(buf, dir, MSG_SIZ);
-	while((q = strchr(p, '/'))) p = q + 1;
-	if(!strncmp(p, "sub_", 4)) {
-	  if(p == buf) safeStrCpy(buf, ".", MSG_SIZ); else p[-1] = NULLCHAR; // strip last directory off path
-	  return LoadSVG(buf, color, piece, retry);
-	}
-      }
       if(!svg && *appData.inscriptions) { // if there is no piece-specific SVG, but we make inscriptions, try general background
 	snprintf(buf, MSG_SIZ, "%s/%sTile.svg", dir, color ? "Black" : "White");
 	svg = rsvg_handle_new_from_file(buf, &svgerror);
@@ -376,7 +352,7 @@ LoadSVG (char *dir, int color, int piece, int retry)
 }
 
 static void
-ScaleOnePiece (int color, int piece, char *pieceDir)
+ScaleOnePiece (int color, int piece)
 {
   float w, h;
   char buf[MSG_SIZ];
@@ -388,11 +364,11 @@ ScaleOnePiece (int color, int piece, char *pieceDir)
   svgPieces[color][piece] = LoadSVG("", color, piece, 0); // this fills pngPieceImages if we had cached svg with bitmap of wanted size
 
   if(!pngPieceImages[color][piece]) { // we don't have cached bitmap (implying we did not have cached svg)
-    if(*pieceDir) { // user specified piece directory
-      snprintf(buf, MSG_SIZ, "%s/%s%s.png", pieceDir, color ? "Black" : "White", pngPieceNames[piece]);
+    if(*appData.pieceDirectory) { // user specified piece directory
+      snprintf(buf, MSG_SIZ, "%s/%s%s.png", appData.pieceDirectory, color ? "Black" : "White", pngPieceNames[piece]);
       img = cairo_image_surface_create_from_png (buf); // try if there are png pieces there
       if(cairo_surface_status(img) != CAIRO_STATUS_SUCCESS) { // there were not
-	svgPieces[color][piece] = LoadSVG(pieceDir, color, piece, 0); // so try if he has svg there
+	svgPieces[color][piece] = LoadSVG(appData.pieceDirectory, color, piece, 0); // so try if he has svg there
       } else pngPieceImages[color][piece] = img;
     }
   }
@@ -425,7 +401,7 @@ ScaleOnePiece (int color, int piece, char *pieceDir)
   cairo_paint (cr);
   cairo_destroy (cr);
 
-  if(!appData.trueColors || !*pieceDir) { // operate on bitmap to color it (king-size hack...)
+  if(!appData.trueColors || !*appData.pieceDirectory) { // operate on bitmap to color it (king-size hack...)
     int stride = cairo_image_surface_get_stride(cs)/4;
     int *buf = (int *) cairo_image_surface_get_data(cs);
     int i, j, p;
@@ -451,12 +427,13 @@ ScaleOnePiece (int color, int piece, char *pieceDir)
 }
 
 void
-CreatePNGPieces (char *pieceDir)
+CreatePNGPieces ()
 {
   int p;
+
   for(p=0; pngPieceNames[p]; p++) {
-    ScaleOnePiece(0, p, pieceDir);
-    ScaleOnePiece(1, p, pieceDir);
+    ScaleOnePiece(0, p);
+    ScaleOnePiece(1, p);
   }
   SelectPieces(gameInfo.variant);
 }
@@ -464,53 +441,25 @@ CreatePNGPieces (char *pieceDir)
 void
 CreateAnyPieces (int p)
 {   // [HGM] taken out of main
-    if(p) CreatePNGPieces(appData.pieceDirectory);
+    if(p) CreatePNGPieces();
     CreatePNGBoard(appData.liteBackTextureFile, 1);
     CreatePNGBoard(appData.darkBackTextureFile, 0);
 }
 
-static void
-ClearPieces ()
+void
+InitDrawingParams (int reloadPieces)
 {
     int i, p;
+    if(reloadPieces)
     for(i=0; i<2; i++) for(p=0; p<BlackPawn; p++) {
 	if(pngPieceImages[i][p]) cairo_surface_destroy(pngPieceImages[i][p]);
 	pngPieceImages[i][p] = NULL;
 	if(svgPieces[i][p]) rsvg_handle_close(svgPieces[i][p], NULL);
 	svgPieces[i][p] = NULL;
     }
-}
-
-void
-InitDrawingParams (int reloadPieces)
-{
-    if(reloadPieces) ClearPieces();
     CreateAnyPieces(1);
 }
 
-void
-Preview (int n, char *s)
-{
-    static Boolean changed[4];
-    changed[n] = TRUE;
-    switch(n) {
-      case 0: // restore true setting
-	if(changed[3]) ClearPieces();
-	CreateAnyPieces(changed[3]); // recomputes textures and (optionally) pieces
-	for(n=0; n<4; n++) changed[n] = FALSE;
-	break;
-      case 1: 
-      case 2:
-	CreatePNGBoard(s, n-1);
-	break;
-      case 3:
-	ClearPieces();
-	CreatePNGPieces(s);
-	break;
-    }
-    DrawPosition(TRUE, NULL);
-}
-
 // [HGM] seekgraph: some low-level drawing routines (by JC, mostly)
 
 float
@@ -537,7 +486,7 @@ void DrawSeekAxis( int x, int y, int xTo, int yTo )
     cairo_t *cr;
 
     /* get a cairo_t */
-    cr = cairo_create (CsBoardWindow(currBoard));
+    cr = cairo_create (csBoardWindow);
 
     cairo_move_to (cr, x, y);
     cairo_line_to(cr, xTo, yTo );
@@ -552,7 +501,7 @@ void DrawSeekAxis( int x, int y, int xTo, int yTo )
 
 void DrawSeekBackground( int left, int top, int right, int bottom )
 {
-    cairo_t *cr = cairo_create (CsBoardWindow(currBoard));
+    cairo_t *cr = cairo_create (csBoardWindow);
 
     cairo_rectangle (cr, left, top, right-left, bottom-top);
 
@@ -566,7 +515,7 @@ void DrawSeekBackground( int left, int top, int right, int bottom )
 
 void DrawSeekText(char *buf, int x, int y)
 {
-    cairo_t *cr = cairo_create (CsBoardWindow(currBoard));
+    cairo_t *cr = cairo_create (csBoardWindow);
 
     cairo_select_font_face (cr, "Sans",
 			    CAIRO_FONT_SLANT_NORMAL,
@@ -585,7 +534,7 @@ void DrawSeekText(char *buf, int x, int y)
 
 void DrawSeekDot(int x, int y, int colorNr)
 {
-    cairo_t *cr = cairo_create (CsBoardWindow(currBoard));
+    cairo_t *cr = cairo_create (csBoardWindow);
     int square = colorNr & 0x80;
     colorNr &= 0x7F;
 
@@ -611,8 +560,7 @@ void DrawSeekDot(int x, int y, int colorNr)
 void
 InitDrawingHandle (Option *opt)
 {
-//    CsBoardWindow = DRAWABLE(opt);
-    currBoard = opt;
+    csBoardWindow = DRAWABLE(opt);
 }
 
 void
@@ -650,7 +598,7 @@ DrawGrid()
   cairo_t *cr;
 
   /* get a cairo_t */
-  cr = cairo_create (CsBoardWindow(currBoard));
+  cr = cairo_create (csBoardWindow);
 
   cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
   SetPen(cr, lineGap, "#000000", 0);
@@ -682,13 +630,13 @@ DrawBorder (int x, int y, int type, int odd)
 	case 2: col = appData.premoveHighlightColor; break;
 	default: col = "#808080"; break; // cannot happen
     }
-    cr = cairo_create(CsBoardWindow(currBoard));
+    cr = cairo_create(csBoardWindow);
     cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
     cairo_rectangle(cr, x+odd/2., y+odd/2., squareSize+lineGap, squareSize+lineGap);
     SetPen(cr, lineGap, col, 0);
     cairo_stroke(cr);
     cairo_destroy(cr);
-//    GraphExpose(currBoard, x - lineGap/2, y - lineGap/2, squareSize+2*lineGap+odd, squareSize+2*lineGap+odd);
+    GraphExpose(currBoard, x - lineGap/2, y - lineGap/2, squareSize+2*lineGap+odd, squareSize+2*lineGap+odd);
 }
 
 static int
@@ -698,7 +646,6 @@ CutOutSquare (int x, int y, int *x0, int *y0, int  kind)
     int nx = x/(squareSize + lineGap), ny = y/(squareSize + lineGap);
     *x0 = 0; *y0 = 0;
     if(textureW[kind] < squareSize || textureH[kind] < squareSize) return 0;
-    if(modV[kind] > 0) nx %= modH[kind], ny %= modV[kind]; // tile fixed-format board periodically to extend it
     if(textureW[kind] < W*squareSize)
 	*x0 = (textureW[kind] - squareSize) * nx/(W-1);
     else
@@ -718,7 +665,7 @@ DrawLogo (Option *opt, void *logo)
     int w, h;
 
     if(!opt) return;
-    cr = cairo_create(CsBoardWindow(opt));
+    cr = cairo_create(DRAWABLE(opt));
     cairo_rectangle (cr, 0, 0, opt->max, opt->value);
     cairo_set_source_rgba(cr, 0.5, 0.5, 0.5, 1.0);
     cairo_fill(cr); // paint background in case logo does not exist
@@ -741,13 +688,18 @@ DrawLogo (Option *opt, void *logo)
 static void
 BlankSquare (cairo_surface_t *dest, int x, int y, int color, ChessSquare piece, int fac)
 {   // [HGM] extra param 'fac' for forcing destination to (0,0) for copying to animation buffer
-    int x0, y0, texture = (useTexture & color+1) && CutOutSquare(x, y, &x0, &y0, color);
+    int x0, y0;
     cairo_t *cr;
 
     cr = cairo_create (dest);
 
-    if(!texture || transparency[color]) // draw color also (as background) when texture could be transparent
-    { // evenly colored squares
+    if ((useTexture & color+1) && CutOutSquare(x, y, &x0, &y0, color)) {
+	    cairo_set_source_surface (cr, pngBoardBitmap[color], x*fac - x0, y*fac - y0);
+	    cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
+	    cairo_rectangle (cr, x*fac, y*fac, squareSize, squareSize);
+	    cairo_fill (cr);
+	    cairo_destroy (cr);
+    } else { // evenly colored squares
 	char *col = NULL;
 	switch (color) {
 	  case 0: col = appData.darkSquareColor; break;
@@ -759,14 +711,8 @@ BlankSquare (cairo_surface_t *dest, int x, int y, int color, ChessSquare piece,
 	cairo_rectangle (cr, fac*x, fac*y, squareSize, squareSize);
 	cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
 	cairo_fill (cr);
+	cairo_destroy (cr);
     }
-    if (texture) {
-	    cairo_set_source_surface (cr, pngBoardBitmap[color], x*fac - x0, y*fac - y0);
-	    cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
-	    cairo_rectangle (cr, x*fac, y*fac, squareSize, squareSize);
-	    cairo_fill (cr);
-    }
-    cairo_destroy (cr);
 }
 
 static void
@@ -781,7 +727,6 @@ pngDrawPiece (cairo_surface_t *dest, ChessSquare piece, int square_color, int x,
 	kind = 1;
 	piece -= BlackPawn;
     }
-    if(piece == WhiteKing && kind == appData.jewelled) piece = WhiteZebra;
     if(appData.upsideDown && flipView) kind = 1 - kind; // swap white and black pieces
     BlankSquare(dest, x, y, square_color, piece, 1); // erase previous contents with background
     cr = cairo_create (dest);
@@ -814,12 +759,12 @@ DoDrawDot (cairo_surface_t *cs, int marker, int x, int y, int r)
 void
 DrawDot (int marker, int x, int y, int r)
 { // used for atomic captures; no need to draw on backup
-  DoDrawDot(CsBoardWindow(currBoard), marker, x, y, r);
+  DoDrawDot(csBoardWindow, marker, x, y, r);
   GraphExpose(currBoard, x-r, y-r, 2*r, 2*r);
 }
 
 static void
-DrawUnicode (cairo_surface_t *canvas, char *string, int x, int y, char id, int flip, int size, int vpos)
+DrawUnicode (cairo_surface_t *canvas, char *string, int x, int y, char id, int flip)
 {
 //	cairo_text_extents_t te;
 	cairo_t *cr;
@@ -832,12 +777,12 @@ DrawUnicode (cairo_surface_t *canvas, char *string, int x, int y, char id, int f
 	cr = cairo_create (canvas);
 	layout = pango_cairo_create_layout(cr);
 	pango_layout_set_text(layout, string, -1);
-	snprintf(fontName, MSG_SIZ, "Sans Normal %dpx", size*squareSize/64);
+	snprintf(fontName, MSG_SIZ, "Sans Normal %dpx", 5*squareSize/8);
 	desc = pango_font_description_from_string(fontName);
 	pango_layout_set_font_description(layout, desc);
 	pango_font_description_free(desc);
         pango_layout_get_pixel_extents(layout, NULL, &r);
-	cairo_translate(cr, x + squareSize/2 - s*r.width/2, y + (32+vpos*s)*squareSize/64 - s*r.height/2);
+	cairo_translate(cr, x + squareSize/2 - s*r.width/2, y + (8+s)*squareSize/16 - s*r.height/2);
 	if(s < 0) cairo_rotate(cr, G_PI);
 	cairo_set_source_rgb(cr, (id == '+' ? 1.0 : 0.0), 0.0, 0.0);
 	pango_cairo_update_layout(cr, layout);
@@ -846,14 +791,14 @@ DrawUnicode (cairo_surface_t *canvas, char *string, int x, int y, char id, int f
 	cairo_destroy(cr);
 }
 
-void
+static void
 DrawText (char *string, int x, int y, int align)
 {
 	int xx = x, yy = y;
 	cairo_text_extents_t te;
 	cairo_t *cr;
 
-	cr = cairo_create (CsBoardWindow(currBoard));
+	cr = cairo_create (csBoardWindow);
 	cairo_select_font_face (cr, "Sans",
 		    CAIRO_FONT_SLANT_NORMAL,
 		    CAIRO_FONT_WEIGHT_BOLD);
@@ -884,15 +829,13 @@ DrawText (char *string, int x, int y, int align)
 void
 InscribeKanji (cairo_surface_t *canvas, ChessSquare piece, int x, int y)
 {
-    char *p, *q, buf[20], nr = 1;
-    int i, n, size = 40, flip = appData.upsideDown && flipView == (piece < BlackPawn);
+    char *p, *q, buf[10];
+    int n, flip = appData.upsideDown && flipView == (piece < BlackPawn);
     if(piece == EmptySquare) return;
     if(piece >= BlackPawn) piece = BLACK_TO_WHITE piece;
     p = appData.inscriptions;
-    if(*p > '0' && *p < '3') nr = *p++ - '0'; // nr of kanji per piece
-    n = piece; i = 0;
+    n = piece;
     while(piece > WhitePawn) {
-      if(*p == '/') p++, piece = n - WhitePBishop; // secondary series
       if(*p++ == NULLCHAR) {
         if(n != WhiteKing) return;
         p = q;
@@ -900,19 +843,12 @@ InscribeKanji (cairo_surface_t *canvas, ChessSquare piece, int x, int y)
       }
       q = p - 1;
       while((*p & 0xC0) == 0x80) p++; // skip UTF-8 continuation bytes
-      if(*q != '.' && ++i < nr) continue; // yet more kanji for the current piece
-      piece--; i = 0;
+      piece--;
     }
-    strncpy(buf, p, 20);
-    for(q=buf; (*++q & 0xC0) == 0x80;); // skip first unicode
-    if(nr > 1) {
-      p = q;
-      while((*++p & 0xC0) == 0x80) {} // skip second unicode
-      *p = NULLCHAR; size = 30; i = 16;
-      DrawUnicode(canvas, q, x, y, PieceToChar(n), flip, size, -10);
-    } else i = 4;
+    strncpy(buf, p, 10);
+    for(q=buf; (*++q & 0xC0) == 0x80;);
     *q = NULLCHAR;
-    DrawUnicode(canvas, buf, x, y, PieceToChar(n), flip, size, i);
+    DrawUnicode(canvas, buf, x, y, PieceToChar(n), flip);
 }
 
 void
@@ -921,10 +857,10 @@ DrawOneSquare (int x, int y, ChessSquare piece, int square_color, int marker, ch
     // piece, background, coordinate/count, marker dot
 
     if (piece == EmptySquare) {
-	BlankSquare(CsBoardWindow(currBoard), x, y, square_color, piece, 1);
+	BlankSquare(csBoardWindow, x, y, square_color, piece, 1);
     } else {
-	pngDrawPiece(CsBoardWindow(currBoard), piece, square_color, x, y);
-        if(appData.inscriptions[0]) InscribeKanji(CsBoardWindow(currBoard), piece, x, y);
+	pngDrawPiece(csBoardWindow, piece, square_color, x, y);
+        if(appData.inscriptions[0]) InscribeKanji(csBoardWindow, piece, x, y);
     }
 
     if(align) { // square carries inscription (coord or piece count)
@@ -933,7 +869,7 @@ DrawOneSquare (int x, int y, ChessSquare piece, int square_color, int marker, ch
     }
 
     if(marker) { // print fat marker dot, if requested
-	DoDrawDot(CsBoardWindow(currBoard), marker, x + squareSize/4, y+squareSize/4, squareSize/2);
+	DoDrawDot(csBoardWindow, marker, x + squareSize/4, y+squareSize/4, squareSize/2);
     }
 }
 
@@ -952,7 +888,7 @@ InitAnimState (AnimNr anr)
 {
     if(c_animBufs[anr]) cairo_surface_destroy (c_animBufs[anr]);
     if(c_animBufs[anr+2]) cairo_surface_destroy (c_animBufs[anr+2]);
-    c_animBufs[anr+4] = CsBoardWindow(currBoard);
+    c_animBufs[anr+4] = csBoardWindow;
     c_animBufs[anr+2] = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, squareSize, squareSize);
     c_animBufs[anr] = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, squareSize, squareSize);
 }
@@ -992,13 +928,13 @@ void CopyRectangle (AnimNr anr, int srcBuf, int destBuf,
 		 int srcX, int srcY, int width, int height, int destX, int destY)
 {
 	cairo_t *cr;
-	c_animBufs[anr+4] = CsBoardWindow(currBoard);
+	c_animBufs[anr+4] = csBoardWindow;
 	cr = cairo_create (c_animBufs[anr+destBuf]);
 	cairo_set_source_surface (cr, c_animBufs[anr+srcBuf], destX - srcX, destY - srcY);
 	cairo_rectangle (cr, destX, destY, width, height);
 	cairo_fill (cr);
 	cairo_destroy (cr);
-	if(c_animBufs[anr+destBuf] == CsBoardWindow(currBoard)) // suspect that GTK needs this!
+	if(c_animBufs[anr+destBuf] == csBoardWindow) // suspect that GTK needs this!
 	    GraphExpose(currBoard, destX, destY, width, height);
 }
 
@@ -1034,7 +970,7 @@ DoDrawPolygon (cairo_surface_t *cs, Pnt arrow[], int nr)
 void
 DrawPolygon (Pnt arrow[], int nr)
 {
-    DoDrawPolygon(CsBoardWindow(currBoard), arrow, nr);
+    DoDrawPolygon(csBoardWindow, arrow, nr);
 //    if(!dual) DoDrawPolygon(csBoardBackup, arrow, nr);
 }
 
@@ -1072,7 +1008,7 @@ DrawSegment (int x, int y, int *lastX, int *lastY, int penType)
   static int curX, curY;
 
   if(penType != PEN_NONE) {
-    cairo_t *cr = cairo_create(CsBoardWindow(disp));
+    cairo_t *cr = cairo_create(DRAWABLE(disp));
     cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
     cairo_move_to (cr, curX, curY);
     cairo_line_to (cr, x,y);
@@ -1091,7 +1027,7 @@ DrawRectangle (int left, int top, int right, int bottom, int side, int style)
 {
   cairo_t *cr;
 
-  cr = cairo_create (CsBoardWindow(disp));
+  cr = cairo_create (DRAWABLE(disp));
   cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
   cairo_rectangle (cr, left, top, right-left, bottom-top);
   switch(side)
@@ -1118,7 +1054,7 @@ DrawEvalText (char *buf, int cbBuf, int y)
 {
     // the magic constants 8 and 5 should really be derived from the font size somehow
   cairo_text_extents_t extents;
-  cairo_t *cr = cairo_create(CsBoardWindow(disp));
+  cairo_t *cr = cairo_create(DRAWABLE(disp));
 
   /* GTK-TODO this has to go into the font-selection */
   cairo_select_font_face (cr, "Sans",
