@@ -5,7 +5,8 @@
  *
  * Copyright 2005 Alessandro Scotti
  *
- * Enhancements Copyright 1995, 2009, 2010, 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+ * Enhancements Copyright 1995, 2009, 2010, 2011, 2012, 2013, 2014,
+ * 2015, 2016 Free Software Foundation, Inc.
  *
  * ------------------------------------------------------------------------
  *
@@ -30,6 +31,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <ctype.h>
 
 #if STDC_HEADERS
 # include <stdlib.h>
@@ -90,9 +92,12 @@ static int  lastForwardMostMove[2] = { -1, -1 };
 static int  engineState[2] = { -1, -1 };
 static char lastLine[2][MSG_SIZ];
 static char header[2][MSG_SIZ];
+static char columnHeader[MSG_SIZ] = "dep\tscore\tnodes\ttime\t(not shown:  tbhits\tknps\tseldep)\n";
+static int  columnMask = 0xF0;
 
 #define MAX_VAR 400
 static int scores[MAX_VAR], textEnd[MAX_VAR], keys[MAX_VAR], curDepth[2], nrVariations[2];
+static char fail[MAX_VAR];
 
 extern int initialRulePlies;
 
@@ -217,25 +222,32 @@ SetProgramStats (FrontEndProgramStats * stats) // now directly called by back-en
         clearMemo = TRUE;
     }
 
-    if( lastForwardMostMove[which] != forwardMostMove ) {
+    if( lastForwardMostMove[which] != forwardMostMove && endPV < 0) {
         clearMemo = TRUE;
     }
 
     if( clearMemo ) {
+        if(!appData.headers) columnHeader[0] = NULLCHAR;
         DoClearMemo(which); nrVariations[which] = 0;
         header[which][0] = NULLCHAR;
         if(gameMode == AnalyzeMode) {
           ChessProgramState *cps = (which ? &second : &first);
-          if((multi = MultiPV(cps)) >= 0) {
-            snprintf(header[which], MSG_SIZ, "\t%s viewpoint\t\tfewer / Multi-PV setting = %d / more\n",
-                                       appData.whitePOV || appData.scoreWhite ? "white" : "mover", cps->option[multi].value);
+          char *exclu = cps->excludeMoves ? exclusionHeader : "";
+          if((multi = MultiPV(cps, 3)) != -1) {
+            char *s = "setting";
+            if(multi < -1) multi = -2 - multi, s = "margin";
+            snprintf(header[which], MSG_SIZ, "\t%s viewpoint\t\tfewer / Multi-PV %s = %d / more\n",
+                                       appData.whitePOV || appData.scoreWhite ? "white" : "mover", s, cps->option[multi].value);
 	  }
-          if(!which) snprintf(header[which]+strlen(header[which]), MSG_SIZ-strlen(header[which]), "%s", exclusionHeader);
+          if(!which) snprintf(header[which]+strlen(header[which]), MSG_SIZ-strlen(header[which]), "%s%s", exclu, columnHeader);
           InsertIntoMemo( which, header[which], 0);
-        } else
-        if(appData.ponderNextMove && lastLine[which][0]) {
+        } else {
+          snprintf(header[which], MSG_SIZ, "%s", columnHeader);
+          if(appData.ponderNextMove && lastLine[which][0]) {
             InsertIntoMemo( which, lastLine[which], 0 );
             InsertIntoMemo( which, "\n", 0 );
+          }
+          InsertIntoMemo( which, header[which], 0);
         }
     }
 
@@ -245,7 +257,7 @@ SetProgramStats (FrontEndProgramStats * stats) // now directly called by back-en
 
     /* Update */
     lastDepth[which] = depth == 1 && ed.nodes == 0 ? 0 : depth; // [HGM] info-line kudge
-    lastForwardMostMove[which] = forwardMostMove;
+    if(endPV < 0) lastForwardMostMove[which] = forwardMostMove; // not during PV walk!
 
     UpdateControls( &ed );
 }
@@ -392,10 +404,19 @@ SetEngineColorIcon (int which)
 
 // [HGM] multivar: sort Thinking Output within one depth on score
 
+static int
+MateFlip (int n)
+{   // map mate-score to monotonous scale, so sorting compares them correctly
+    if(n >=  MATE_SCORE) return 2*MATE_SCORE - n;
+    if(n <= -MATE_SCORE) return -2*MATE_SCORE - n;
+    return n;
+}
+
 static int
 InsertionPoint (int len, EngineOutputData *ed)
 {
 	int i, offs = 0, newScore = ed->score, n = ed->which;
+	char failType;
 
 	if(ed->nodes == 0 && ed->score == 0 && ed->time == 0)
 		newScore = 1e6; // info lines inserted on top
@@ -403,6 +424,9 @@ InsertionPoint (int len, EngineOutputData *ed)
 		curDepth[n] = ed->depth;
 		nrVariations[n] = 0; // throw away everything we had
 	}
+	i = strlen(ed->pv); if(i > 0) i--;
+	failType = ed->pv[i];
+	if(failType != '?' && failType != '!') failType = ' ';
 	// loop through all lines. Note even / odd used for different panes
 	for(i=nrVariations[n]-2; i>=0; i-=2) {
 		// put new item behind those we haven't looked at
@@ -410,22 +434,42 @@ InsertionPoint (int len, EngineOutputData *ed)
 		textEnd[i+n+2] = offs + len;
 		scores[i+n+2] = newScore;
 		keys[i+n+2] = ed->moveKey;
+		fail[i+n+2] = failType;
 		if(ed->moveKey != keys[i+n] && // same move always tops previous one (as a higher score must be a fail low)
-		   newScore < scores[i+n]) break;
+		   MateFlip(newScore) < MateFlip(scores[i+n]) && fail[i+n] == ' ') break;
 		// if it had higher score as previous, move previous in stead
 		scores[i+n+2] = ed->moveKey == keys[i+n] ? newScore : scores[i+n]; // correct scores of fail-low/high searches
 		textEnd[i+n+2] = textEnd[i+n] + len;
 		keys[i+n+2] = keys[i+n];
+		fail[i+n+2] = fail[i+n];
 	}
 	if(i<0) {
 		offs = 0;
 		textEnd[n] = offs + len;
 		scores[n] = newScore;
+		keys[n] = ed->moveKey;
+		fail[n] = failType;
 	}
 	nrVariations[n] += 2;
-      return offs + (gameMode == AnalyzeMode)*strlen(header[ed->which]);
+      return offs + strlen(header[ed->which]);
 }
 
+static char spaces[] = "            "; // [HGM] align: spaces for padding
+
+static void
+Format(char *buf, int val)
+{ // [HGM] tbhits: print a positive integer with trailing whitespace to give it fixed width
+        if( val < 1000000 ) {
+            int h = val, i=0;
+            while(h > 0) h /= 10, i++;
+            snprintf( buf, 24, "%d%s\t", val, spaces + 2*i);
+        }
+        else {
+            char unit = 'M';
+            if(val >= 1e9) val /= 1e3, unit = 'G';
+            snprintf( buf, 24, "%.*f%c%s\t", 1 + (val < 1e7), val/1e6, unit, spaces + 10 + 2*(val >= 1e8));
+        }
+}
 
 // pure back end, now SetWindowText is called via wrapper DoSetWindowText
 static void
@@ -436,6 +480,7 @@ UpdateControls (EngineOutputData *ed)
     char s_label[MAX_NAME_LENGTH + 32];
     int h;
     char * name = ed->name;
+    char *q, *pvStart = ed->pv;
 
     /* Label */
     if( name == 0 || *name == '\0' ) {
@@ -445,9 +490,17 @@ UpdateControls (EngineOutputData *ed)
     strncpy( s_label, name, MAX_NAME_LENGTH );
     s_label[ MAX_NAME_LENGTH-1 ] = '\0';
 
+    if(pvStart) { // [HGM] tbhits: plit up old PV into extra infos and real PV
+        while(strchr(pvStart, '\t')) { // locate last tab before non-int (real PV starts after that)
+            for(q=pvStart; isdigit(*q) || *q == ' '; q++);
+            if(*q != '\t') break;
+            pvStart = q + 1;
+        }
+    }
+
 #ifdef SHOW_PONDERING
     if( IsEnginePondering( ed->which ) ) {
-        char buf[8];
+        char buf[12];
 
         buf[0] = '\0';
 
@@ -455,16 +508,17 @@ UpdateControls (EngineOutputData *ed)
             strncpy( buf, ed->hint, sizeof(buf) );
             buf[sizeof(buf)-1] = '\0';
         }
-        else if( ed->pv != 0 && *ed->pv != '\0' ) {
-            char * sep = strchr( ed->pv, ' ' );
+        else if( pvStart != 0 && *pvStart != '\0' ) {
+            char * sep;
             int buflen = sizeof(buf);
 
+            sep = strchr( pvStart, ' ' );
             if( sep != NULL ) {
-                buflen = sep - ed->pv + 1;
+                buflen = sep - pvStart + 1;
                 if( buflen > sizeof(buf) ) buflen = sizeof(buf);
             }
 
-            strncpy( buf, ed->pv, buflen );
+            strncpy( buf, pvStart, buflen );
             buf[ buflen-1 ] = '\0';
         }
 
@@ -516,45 +570,83 @@ UpdateControls (EngineOutputData *ed)
     DoSetWindowText( ed->which, nLabelNPS, s_label );
 
     /* Memo */
-    if( ed->pv != 0 && *ed->pv != '\0' ) {
+    if( pvStart != 0 && *pvStart != '\0' ) {
         char s_nodes[24];
         char s_score[16];
         char s_time[24];
-        char buf[256];
-        int buflen;
+        char s_hits[24];
+        char s_seld[24];
+        char s_knps[24];
+        char buf[256], fail;
+        int buflen, hits, i, params[5], extra;
         int time_secs = ed->time / 100;
         int time_cent = ed->time % 100;
 
         /* Nodes */
         if( ed->nodes < 1000000 ) {
-            snprintf( s_nodes, sizeof(s_nodes)/sizeof(s_nodes[0]), u64Display, ed->nodes );
+            int h = ed->nodes, i=0;
+            while(h > 0) h /= 10, i++; // [HGM] align: count digits; pad with 2 spaces for every missing digit
+            snprintf( s_nodes, sizeof(s_nodes)/sizeof(s_nodes[0]), u64Display "%s\t", ed->nodes, spaces + 2*i);
         }
         else {
-            snprintf( s_nodes, sizeof(s_nodes)/sizeof(s_nodes[0]), "%.1fM", u64ToDouble(ed->nodes) / 1000000.0 );
+            double x = u64ToDouble(ed->nodes);
+            char unit = 'M';
+            if(x >= 1e9) x /= 1e3, unit = 'G';
+            snprintf( s_nodes, sizeof(s_nodes)/sizeof(s_nodes[0]), "%.*f%c%s\t", 1 + (x < 1e7), x / 1e6,
+                      unit, spaces + 10 + 2*(ed->nodes >= 1e8));
         }
 
+        /* TB Hits etc. */
+        for(i=hits=0; i<5; i++) params[i] = 0;
+//fprintf(stderr, "%s\n%s\n", ed->pv, pvStart);
+        if(pvStart != ed->pv) { // check if numbers before PV
+            strncpy(buf, ed->pv, 256); buf[pvStart - ed->pv] = NULLCHAR;
+            extra = sscanf(buf, "%d %d %d %d %d", params, params+1, params+2, params+3, params+4);
+//fprintf(stderr, "extra=%d len=%d\n", extra, pvStart - ed->pv);
+            if(extra) hits = params[extra-1], params[extra-1] = 0; // last one is tbhits
+        }
+        Format(s_seld, params[0]); Format(s_knps, params[1]); Format(s_hits, hits); 
+
+        if(*ed->pv) fail = ed->pv[strlen(ed->pv)-1]; else fail = ' ';
+	if(fail != '?' && fail != '!') fail = ' ';
+
         /* Score */
         h = ((gameMode == AnalyzeMode && appData.whitePOV || appData.scoreWhite) && !WhiteOnMove(currentMove) ? -1 : 1) * ed->score;
+        if( h == 0 ) {
+	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "  0.00%c\t", fail );
+        } else
+	if( h >= MATE_SCORE) snprintf(s_score, 16, "  %s#%d%c\t", ( h > MATE_SCORE+9 ? "" : "  "),  h - MATE_SCORE, fail ); else
+	if(-h >= MATE_SCORE) snprintf(s_score, 16, " %s#-%d%c\t", (-h > MATE_SCORE+9 ? "" : "  "), -h - MATE_SCORE, fail ); else
         if( h > 0 ) {
-	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "+%.2f", h / 100.0 );
+	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "+%.2f%c\t", h / 100.0, fail );
         }
         else {
-	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "%.2f", h / 100.0 );
+	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), " %.2f%c\t", h / 100.0, fail );
         }
 
         /* Time */
-        snprintf( s_time, sizeof(s_time)/sizeof(s_time[0]), "%d:%02d.%02d", time_secs / 60, time_secs % 60, time_cent );
+        if(time_secs >= 3600)
+            snprintf( s_time, sizeof(s_time)/sizeof(s_time[0]), "%d:%02d:%02d\t", time_secs / 3600, (time_secs / 60) % 60, time_secs % 60 );
+        else
+        snprintf( s_time, sizeof(s_time)/sizeof(s_time[0]), "%d:%02d.%02d\t", time_secs / 60, time_secs % 60, time_cent );
+
+        if(columnMask & 2) s_score[0] = NULLCHAR; // [HGM] hide: erase columns the user has hidden
+        if(columnMask & 4) s_nodes[0] = NULLCHAR;
+        if(columnMask & 8) s_time[0]  = NULLCHAR;
+        if(columnMask & 16) s_hits[0]  = NULLCHAR;
+        if(columnMask & 32) s_knps[0]  = NULLCHAR;
+        if(columnMask & 64) s_seld[0]  = NULLCHAR;
 
         /* Put all together... */
 	if(ed->nodes == 0 && ed->score == 0 && ed->time == 0)
 	  snprintf( buf, sizeof(buf)/sizeof(buf[0]), "%3d\t", ed->depth );
 	else
-	  snprintf( buf, sizeof(buf)/sizeof(buf[0]), "%3d\t%s\t%s\t%s\t", ed->depth, s_score, s_nodes, s_time );
+	  snprintf( buf, sizeof(buf)/sizeof(buf[0]), "%3d\t%s%s%s%s%s%s", ed->depth, s_score, s_nodes, s_time, s_hits, s_knps, s_seld );
 
         /* Add PV */
         buflen = strlen(buf);
 
-        strncpy( buf + buflen, ed->pv, sizeof(buf) - buflen );
+        strncpy( buf + buflen, pvStart, sizeof(buf) - buflen );
 
         buf[ sizeof(buf) - 3 ] = '\0';
 
@@ -569,6 +661,35 @@ UpdateControls (EngineOutputData *ed)
     SetEngineColorIcon( ed->which );
 }
 
+static char *titles[] = { "score\t", "nodes\t", "time\t", "tbhits\t", "knps\t", "seldep\t" };
+
+void
+Collapse(int n)
+{   // handle click on column headers, to hide / show them
+    int i, j, nr=0, m=~columnMask, Ncol=7;
+    for(i=0; columnHeader[i] && i<n; i++) nr += (columnHeader[i] == '\t');
+    if(!nr) return; // depth always shown, so clicks on it ignored
+    for(i=j=0; i<Ncol; i++) if(m & 1<<i) j++; // count hidden columns
+    if(nr < j) { // shown column clicked: hide it
+	for(i=j=0; i<Ncol; i++) if(m & 1<<i && j++ == nr) break;
+	columnMask |= 1<<i;
+    } else { // hidden column clicked: show it
+	m = ~m; nr -= j;
+	for(i=j=0; i<Ncol; i++) if(m & 1<<i && j++ == nr) break;
+	columnMask &= ~(1<<i);
+    }
+    // create new header line
+    strcpy(columnHeader, "dep\t");
+    m = ~columnMask;
+    for(i=j=1; i<Ncol; i++) if(m & 1<<i) strcat(columnHeader, titles[i-1]), j++;
+    if(j != Ncol) { // list hidden columns, so user ca click them
+	m = ~m; strcat(columnHeader, "(not shown:  ");
+	for(i=1; i<Ncol; i++) if(m & 1<<i) strcat(columnHeader, titles[i-1]);
+	strcat(columnHeader, ")");
+    }
+    strcat(columnHeader, "\n");
+}
+
 // [HGM] kibitz: write kibitz line; split window for it if necessary
 void
 OutputKibitz (int window, char *text)
@@ -582,7 +703,7 @@ OutputKibitz (int window, char *text)
 	}
 	opponentKibitzes = TRUE; // this causes split window DisplayMode in ICS modes.
 	VerifyDisplayMode();
-	strncpy(text+strlen(text)-1, "\r\n",sizeof(text+strlen(text)-1)); // to not lose line breaks on copying
+	strncpy(text+strlen(text)-1, "\r\n", 4); // to not lose line breaks on copying
 	if(gameMode == IcsObserving) {
 	    DoSetWindowText(0, nLabel, gameInfo.white);
 	    SetIcon( 0, nColorIcon,  nColorWhite);
