@@ -175,12 +175,18 @@ extern char *getenv();
 #  define IMG ".png"
    // redefine some defaults
 #  undef ICS_LOGON
+#  undef DATADIR
+#  undef MANDIR
 #  undef LOCALEDIR
 #  undef SETTINGS_FILE
 #  define ICS_LOGON "Library/Preferences/XboardICS.conf"
+#  define DATADIR dataDir
+#  define MANDIR manDir
 #  define LOCALEDIR localeDir
 #  define SETTINGS_FILE masterSettings
 #  define SYNC_MENUBAR gtkosx_application_sync_menubar(theApp)
+   char dataDir[MSG_SIZ]; // for expanding ~~
+   char manDir[MSG_SIZ];
    char localeDir[MSG_SIZ];
    char masterSettings[MSG_SIZ];
 #else
@@ -256,7 +262,6 @@ GtkAccelGroup *GtkAccelerators;
 typedef unsigned int BoardSize;
 BoardSize boardSize;
 Boolean chessProgram;
-int initialSquareSize;
 
 int  minX, minY; // [HGM] placement: volatile limits on upper-left corner
 int smallLayout = 0, tinyLayout = 0,
@@ -381,6 +386,7 @@ colorVariable[] = {
 
 // [HGM] font: keep a font for each square size, even non-stndard ones
 #define NUM_SIZES 18
+#define MAX_SIZE 130
 Boolean fontIsSet[NUM_FONTS], fontValid[NUM_FONTS][MAX_SIZE];
 char *fontTable[NUM_FONTS][MAX_SIZE];
 
@@ -442,17 +448,6 @@ SetFontDefaults ()
   appData.gameListFont = strdup(GAMELIST_FONT_NAME);
 }
 
-void
-ChangeFont (int force, char **font, int fnr, int size, char *def, int pix)
-{
-    if(!fontValid[fnr][size]) {
-	if(fontIsSet[fnr] && !force) return; // unless forced we do not replace an explicitly specified font by a default
-	ASSIGN(fontTable[fnr][size], def);   // use default
-	fontIsSet[fnr] = False;
-    } else fontIsSet[fnr] = True;
-    FREE(*font); *font = InsertPxlSize(fontTable[fnr][size], pix);
-}
-
 void
 CreateFonts ()
 { // no-op, until we identify the code for this already in XBoard and move it here
@@ -526,11 +521,10 @@ SaveFontArg (FILE *f, ArgDescriptor *ad)
     default:
       return;
   }
-  if(fontIsSet[n])           // only save fonts that were not defaults
   for(i=0; i<NUM_SIZES; i++) // [HGM] font: current font becomes standard for current size
-    if(sizeDefaults[i].squareSize == initialSquareSize) { // only for standard sizes!
-	ASSIGN(fontTable[n][initialSquareSize], name);
-	fontValid[n][initialSquareSize] = True;
+    if(sizeDefaults[i].squareSize == squareSize) { // only for standard sizes!
+	fontTable[n][squareSize] = strdup(name);
+	fontValid[n][squareSize] = True;
 	break;
   }
   for(i=0; i<MAX_SIZE; i++) if(fontValid[n][i]) // [HGM] font: store all standard fonts
@@ -664,9 +658,9 @@ ResizeBoardWindow (int w, int h, int inhibit)
     gtk_widget_set_size_request(optList[W_BOARD].handle, w, h);   // protect board widget
 //    w += marginW + 1; // [HGM] not sure why the +1 is (sometimes) needed...
 //    h += marginH + a.height + 1;
-    gtk_window_resize(GTK_WINDOW(shellWidget), w, 10);
+    gtk_window_resize(GTK_WINDOW(shellWidget), w, h);
     DoEvents();
-    if(!appData.fixedSize) gtk_widget_set_size_request(optList[W_BOARD].handle, 100, 100); // liberate board again
+    gtk_widget_set_size_request(optList[W_BOARD].handle, -1, -1); // liberate board again
 }
 
 int
@@ -680,21 +674,21 @@ InitializeFonts (int clockFontPxlSize, int coordFontPxlSize, int fontPxlSize)
 {   // determine what fonts to use, and create them
 
     if(!fontIsSet[CLOCK_FONT] && fontValid[CLOCK_FONT][squareSize])
-	appData.clockFont = fontTable[CLOCK_FONT][squareSize], fontIsSet[CLOCK_FONT] = True;
+	appData.clockFont = fontTable[CLOCK_FONT][squareSize];
     if(!fontIsSet[MESSAGE_FONT] && fontValid[MESSAGE_FONT][squareSize])
-	appData.font = fontTable[MESSAGE_FONT][squareSize], fontIsSet[MESSAGE_FONT] = True;
+	appData.font = fontTable[MESSAGE_FONT][squareSize];
     if(!fontIsSet[COORD_FONT] && fontValid[COORD_FONT][squareSize])
-	appData.coordFont = fontTable[COORD_FONT][squareSize], fontIsSet[COORD_FONT] = True;
+	appData.coordFont = fontTable[COORD_FONT][squareSize];
     if(!fontIsSet[CONSOLE_FONT] && fontValid[CONSOLE_FONT][squareSize])
-	appData.icsFont = fontTable[CONSOLE_FONT][squareSize], fontIsSet[CONSOLE_FONT] = True;
+	appData.icsFont = fontTable[CONSOLE_FONT][squareSize];
     if(!fontIsSet[EDITTAGS_FONT] && fontValid[EDITTAGS_FONT][squareSize])
-	appData.tagsFont = fontTable[EDITTAGS_FONT][squareSize], fontIsSet[EDITTAGS_FONT] = True;
+	appData.tagsFont = fontTable[EDITTAGS_FONT][squareSize];
     if(!fontIsSet[COMMENT_FONT] && fontValid[COMMENT_FONT][squareSize])
-	appData.commentFont = fontTable[COMMENT_FONT][squareSize], fontIsSet[COMMENT_FONT] = True;
+	appData.commentFont = fontTable[COMMENT_FONT][squareSize];
     if(!fontIsSet[MOVEHISTORY_FONT] && fontValid[MOVEHISTORY_FONT][squareSize])
-	appData.historyFont = fontTable[MOVEHISTORY_FONT][squareSize], fontIsSet[MOVEHISTORY_FONT] = True;
+	appData.historyFont = fontTable[MOVEHISTORY_FONT][squareSize];
     if(!fontIsSet[GAMELIST_FONT] && fontValid[GAMELIST_FONT][squareSize])
-	appData.gameListFont = fontTable[GAMELIST_FONT][squareSize], fontIsSet[GAMELIST_FONT] = True;
+	appData.gameListFont = fontTable[GAMELIST_FONT][squareSize];
 
     appData.font = InsertPxlSize(appData.font, coordFontPxlSize);
     appData.clockFont = InsertPxlSize(appData.clockFont, clockFontPxlSize);
@@ -925,8 +919,7 @@ main (int argc, char **argv)
     if(argc > 1 && !strcmp(argv[1], "--show-config")) { // [HGM] install: called to print config info
 	typedef struct {char *name, *value; } Config;
 	static Config configList[] = {
-	  { "Datadir", dataDir },
-	  { "Mandir", manDir },
+	  { "Datadir", DATADIR },
 	  { "Sysconfdir", SYSCONFDIR },
 	  { NULL }
 	};
@@ -971,7 +964,7 @@ main (int argc, char **argv)
 
     { // [HGM] initstring: kludge to fix bad bug. expand '\n' characters in init string and computer string.
 	static char buf[MSG_SIZ];
-	snprintf(buf, MSG_SIZ, appData.sysOpen, dataDir);
+	snprintf(buf, MSG_SIZ, appData.sysOpen, DATADIR);
 	ASSIGN(appData.sysOpen, buf); // expand %s in -openCommand to DATADIR (usefull for OS X configuring)
 	EscapeExpand(buf, appData.firstInitString);
 	appData.firstInitString = strdup(buf);
@@ -984,9 +977,7 @@ main (int argc, char **argv)
     }
 
     if ((chessDir = (char *) getenv("CHESSDIR")) == NULL) {
-	static char dirName[MSG_SIZ];
-	getcwd(dirName, MSG_SIZ);
-	chessDir = dirName;
+	chessDir = ".";
     } else {
 	if (chdir(chessDir) != 0) {
 	    fprintf(stderr, _("%s: can't cd to CHESSDIR: "), programName);
@@ -1087,7 +1078,6 @@ main (int argc, char **argv)
 	tinyLayout = szd->tinyLayout;
 	// [HGM] font: use defaults from settings file if available and not overruled
     }
-    initialSquareSize = squareSize; // [HGM] remember for saving font info
     if(BOARD_WIDTH != 8) {
 	squareSize = (squareSize*8 + BOARD_WIDTH/2)/BOARD_WIDTH; // keep width the same
 	lineGap = (squareSize < 37 ? 1 : squareSize < 59 ? 2 : squareSize < 116 ? 3 : 4);
@@ -1154,7 +1144,6 @@ main (int argc, char **argv)
     menuBarWidget    = optList[W_MENU].handle;
     dropMenu         = optList[W_DROP].handle;
     titleWidget = optList[optList[W_TITLE].type != Skip ? W_TITLE : W_SMALL].handle;
-    DelayedDrag(); // fake configure event (i3wm tiling window manager fails to send one after initial resize)
 #ifdef TODO_GTK
     formWidget  = XtParent(boardWidget);
     XtSetArg(args[0], XtNbackground, &timerBackgroundPixel);
@@ -1676,8 +1665,9 @@ void
 ReSize (WindowPlacement *wp)
 {
 	GtkAllocation a;
-	int sqx, sqy, i, w, h, lg = lineGap;
-	if(wp->width == wpMain.width && wp->height == wpMain.height) return; // not sized
+	int sqx, sqy, w, h, lg = lineGap;
+	static int first = 1;
+	if(wp->width == wpMain.width && wp->height == wpMain.height && !first) return; // not sized
 	gtk_widget_get_allocation(optList[W_DROP+1].handle, &a); // table that should contain everything
 	w = a.width; h = a.height;
 	gtk_widget_get_allocation(shellWidget, &a);
@@ -1702,56 +1692,16 @@ ReSize (WindowPlacement *wp)
 	    lg = sqx < 37 ? 1 : sqx < 59 ? 2 : sqx < 116 ? 3 : 4;
 	    if(sqx == oldSqx + 1 && lg == lineGap + 1) sqx = oldSqx, squareSize = 0; // prevent oscillations, force resize by kludge
 	}
-	for(h=0; sizeDefaults[h+1].name && sizeDefaults[h].squareSize*8 > sqx*BOARD_WIDTH; h++) {}
-	if(initialSquareSize != sizeDefaults[h].squareSize && !appData.fixedSize) { // boardSize changed
-	    initialSquareSize = sizeDefaults[h].squareSize; // used for saving font
-	    ChangeFont(1, &appData.clockFont, CLOCK_FONT, initialSquareSize, CLOCK_FONT_NAME, 2*(sizeDefaults[h].clockFontPxlSize+1)/3);
-	    ChangeFont(1, &appData.font, MESSAGE_FONT, initialSquareSize, DEFAULT_FONT_NAME, sizeDefaults[h].coordFontPxlSize);
-	    ChangeFont(0, &appData.icsFont, CONSOLE_FONT, initialSquareSize, CONSOLE_FONT_NAME, sizeDefaults[h].coordFontPxlSize);
-	    ChangeFont(0, &appData.tagsFont, EDITTAGS_FONT, initialSquareSize, TAGS_FONT_NAME, sizeDefaults[h].coordFontPxlSize);
-	    ChangeFont(0, &appData.commentFont, COMMENT_FONT, initialSquareSize, COMMENT_FONT_NAME, sizeDefaults[h].coordFontPxlSize);
-	    ChangeFont(0, &appData.gameListFont, GAMELIST_FONT, initialSquareSize, GAMELIST_FONT_NAME, sizeDefaults[h].coordFontPxlSize);
-	    ChangeFont(0, &appData.historyFont, MOVEHISTORY_FONT, initialSquareSize, HISTORY_FONT_NAME, sizeDefaults[h].coordFontPxlSize);
-	    DisplayBothClocks();
-	    ApplyFont(&mainOptions[W_MESSG], NULL);
-	    for(i=1; i<6; i++) ApplyFont(&mainOptions[W_BUTTON+i], NULL);
-	    ApplyFont(&tagsOptions[1], NULL);
-	    ApplyFont(&commentOptions[0], NULL);
-	    ApplyFont(&historyOptions[0], NULL);
-	    ApplyFont(&engoutOptions[5], NULL);
-	    ApplyFont(&engoutOptions[12], NULL);
-	    ApplyFont(&chatOptions[11], appData.icsFont);
-	    AppendColorized(&chatOptions[6], NULL, 0); // kludge to replace font tag
-	}
-	if(!strchr(appData.boardSize, ',')) {
-	    ASSIGN(appData.boardSize, sizeDefaults[h].name);
-	}
-#ifndef OSXAPP
-	if(sizeDefaults[h].tinyLayout != tinyLayout) { // alter clipping of menu names to conform to board width
-	    int clip = (tinyLayout = sizeDefaults[h].tinyLayout) + 1;
-	    char text[MSG_SIZ];
-	    for(h=1; mainOptions[h].type == DropDown; h++) {
-		strncpy(text, _(mainOptions[h].name), MSG_SIZ);
-		if(clip != 1) text[clip + (text[clip-1] == '_')] = NULLCHAR;
-		gtk_menu_item_set_label((GtkMenuItem *) mainOptions[h].handle, text);
-	    }
-	}
-#endif
-	if(sqx != squareSize && !appData.fixedSize) {
+	if(sqx != squareSize && !first) {
 	    squareSize = sqx; // adopt new square size
-	    CreatePNGPieces(appData.pieceDirectory); // make newly scaled pieces
+	    CreatePNGPieces(); // make newly scaled pieces
 	    InitDrawingSizes(0, 0); // creates grid etc.
 	} else ResizeBoardWindow(BOARD_WIDTH * (squareSize + lineGap) + lineGap, BOARD_HEIGHT * (squareSize + lineGap) + lineGap, 0);
 	w = BOARD_WIDTH * (squareSize + lineGap) + lineGap;
 	h = BOARD_HEIGHT * (squareSize + lineGap) + lineGap;
 	if(optList[W_BOARD].max   > w) optList[W_BOARD].max = w;
 	if(optList[W_BOARD].value > h) optList[W_BOARD].value = h;
-	if(twoBoards && shellUp[DummyDlg]) {
-	    SlavePopUp(); dualOptions[3].max = 0; DoEvents(); // calls SlaveResize, kludge to force assigning new canvas
-	    partnerUp = !partnerUp; flipView = !flipView;
-	    DrawPosition(True, NULL);
-	    partnerUp = !partnerUp; flipView = !flipView;
-	}
+	first = appData.fixedSize;
 }
 
 static guint delayedDragTag = 0;
@@ -1759,19 +1709,16 @@ static guint delayedDragTag = 0;
 void
 DragProc ()
 {
-    static int busy;
-    if(busy++) return; // prevent recursive calling, but remember we missed an event in 'busy'
+	static int busy;
+	if(busy) return;
 
-    if(delayedDragTag) g_source_remove(delayedDragTag); // no more timer interrupts from same event!
-    delayedDragTag = 0;
-
-    do {
+	busy = 1;
 	GetActualPlacement(shellWidget, &wpNew);
 	if(wpNew.x == wpMain.x && wpNew.y == wpMain.y && // not moved
 	   wpNew.width == wpMain.width && wpNew.height == wpMain.height) { // not sized
-	    busy = 0; break; // false alarm
+	    busy = 0; return; // false alarm
 	}
-	ReSize(&wpNew); // this can be interrupted by other events
+	ReSize(&wpNew);
 	if(appData.useStickyWindows) {
 	    if(shellUp[EngOutDlg]) CoDrag(shells[EngOutDlg], &wpEngineOutput);
 	    if(shellUp[HistoryDlg]) CoDrag(shells[HistoryDlg], &wpMoveHistory);
@@ -1781,8 +1728,9 @@ DragProc ()
         }
 	wpMain = wpNew;
 	DrawPosition(True, NULL);
-	if(busy > 2) busy = 2; // if multiple events were backlogged, only do one more
-    } while(--busy);
+	if(delayedDragTag) g_source_remove(delayedDragTag);
+	delayedDragTag = 0; // now drag executed, make sure next DelayedDrag will not cancel timer event (which could now be used by other)
+	busy = 0;
 }
 
 void
@@ -1803,7 +1751,6 @@ EventProc (GtkWidget *widget, GdkEvent *event, gpointer g)
     wpNew.y = event->configure.y;
     wpNew.width  = event->configure.width;
     wpNew.height = event->configure.height;
-//    SetWidgetLabel(&mainOptions[W_WHITE], ""); SetWidgetLabel(&mainOptions[W_BLACK], "");
     DelayedDrag(); // as long as events keep coming in faster than 50 msec, they destroy each other
     return FALSE;
 }
@@ -2107,33 +2054,13 @@ DisplayIcsInteractionTitle (String message)
 #endif
 }
 
-void
-LockBoardSize (int after)
-{
-    static char *oldClockFont, *oldMessgFont;
-    int w, h;
-    if(oldMessgFont && !strcmp(oldMessgFont, appData.font) &&
-       oldClockFont && !strcmp(oldClockFont, appData.clockFont) && after < 2) return; // only do something when font changed
-    w = BOARD_WIDTH*(squareSize + lineGap) + lineGap;
-    h = BOARD_HEIGHT*(squareSize + lineGap) + lineGap;
-    if(after & 1) {
-	ASSIGN(oldClockFont, appData.clockFont);
-	ASSIGN(oldMessgFont, appData.font);
-	gtk_window_resize(GTK_WINDOW(shellWidget), w, h);
-	DoEvents();
-	gtk_widget_set_size_request(optList[W_BOARD].handle, -1, -1); // liberate board
-    } else { // before
-	gtk_widget_set_size_request(optList[W_BOARD].handle, w, h);   // protect board widget
-    }
-}
 
 void
 DisplayTimerLabel (Option *opt, char *color, long timer, int highlight)
 {
-    static int twoLines = -1;
     GtkWidget *w = (GtkWidget *) opt->handle;
     GdkColor col;
-    char *markup, two = (appData.logoSize != 0);
+    char *markup;
     char bgcolor[10];
     char fgcolor[10];
 
@@ -2150,14 +2077,12 @@ DisplayTimerLabel (Option *opt, char *color, long timer, int highlight)
         strcpy(fgcolor, appData.lowTimeWarningColor);
     }
 
-    if(! partnerUp && two != twoLines) LockBoardSize(2); // lock board size if clock height changes
-
     gdk_color_parse( bgcolor, &col );
     gtk_widget_modify_bg(gtk_widget_get_parent(opt->handle), GTK_STATE_NORMAL, &col);
 
     if (appData.clockMode) {
         markup = g_markup_printf_escaped("<span font=\"%s\" background=\"%s\" foreground=\"%s\">%s:%s%s</span>", appData.clockFont,
-					 bgcolor, fgcolor, color, two ? "\n" : " ", TimeString(timer));
+					 bgcolor, fgcolor, color, appData.logoSize && !partnerUp ? "\n" : " ", TimeString(timer));
 //        markup = g_markup_printf_escaped("<span size=\"xx-large\" weight=\"heavy\" background=\"%s\" foreground=\"%s\">%s:%s%s</span>",
 //					 bgcolor, fgcolor, color, appData.logoSize && !partnerUp ? "\n" : " ", TimeString(timer));
     } else {
@@ -2168,8 +2093,6 @@ DisplayTimerLabel (Option *opt, char *color, long timer, int highlight)
     }
     gtk_label_set_markup(GTK_LABEL(w), markup);
     g_free(markup);
-
-    if(!partnerUp && two != twoLines) LockBoardSize(3), twoLines = two;
 }
 
 static GdkPixbuf **clockIcons[] = { &WhiteIcon, &BlackIcon };
@@ -2403,15 +2326,15 @@ UpdateLogos (int displ)
     return;
 }
 
-void FileNamePopUpWrapper(
-     char *label,
-     char *def,
-     char *filter,
-     FileProc proc,
-     Boolean pathFlag,
-     char *openMode,
-     char **name,
-     FILE **fp)
+void FileNamePopUpWrapper(label, def, filter, proc, pathFlag, openMode, name, fp)
+     char *label;
+     char *def;
+     char *filter;
+     FileProc proc;
+     char *openMode;
+     Boolean pathFlag;
+     char **name;
+     FILE **fp;
 {
   GtkWidget     *dialog;
   GtkFileFilter *gtkfilter;
